%bcond_with texlive
%bcond_with check

%global blaslib flexiblas

%global somajor %%(echo %{version} | cut -d. -f1)
%global soversion %{somajor}.1
%global __requires_exclude libHYPRE.*
%{?filter_setup:
%filter_from_requires /libHYPRE/d
%filter_setup
}

%global lalibs --with-blas-libs=%{blaslib} --with-lapack-libs=%{blaslib} \\\
           --with-blas-lib-dirs=%_libdir --with-lapack-lib-dirs=%_libdir

Summary:        High performance matrix preconditioners
Name:           hypre
Version:        2.24.0
Release:        4%{?dist}
License:        LGPLv2
URL:            http://www.llnl.gov/casc/hypre/
Source0:        https://github.com/hypre-space/hypre/archive/v%version/%{name}-%{version}.tar.gz

Patch3000:      hypre-test.patch

BuildRequires:  gcc-c++ gcc-gfortran automake libtool libtool-ltdl-devel SuperLU-devel %{blaslib}-devel
%if %{with texlive}
BuildRequires:  doxygen-latex python-sphinx python-sphinx-theme-alabaster python3-breathe python%3-sphinx-latex
BuildRequires: /usr/bin/latexmk tex(threeparttable.sty) tex(hanging.sty) tex(adjustbox.sty) tex(fncychap.sty)
BuildRequires: tex(tabulary.sty) tex(capt-of.sty) tex(needspace.sty) tex(stackengine.sty) tex(listofitems.sty)
BuildRequires: tex(ulem.sty) tex(etoc.sty)
%endif

%description
Hypre is a set of matrix preconditioning libraries to aid in the solution of large systems of linear equations.

%package devel
Summary:        Development files for %name
Requires:       %{?name} = %{version}-%{release} SuperLU-devel %{blaslib}-devel

%description devel
Development files for %name

%package openmpi
Summary:        High performance matrix preconditioners - openmpi
BuildRequires:  superlu_dist-openmpi-devel ptscotch-openmpi-devel
Requires:       openmpi

%description openmpi
Hypre is a set of matrix preconditioning libraries to aid in the solution of large systems of linear equations.

This is the openmpi version.

%package openmpi-devel
Summary:        Development files for %name-openmpi
Requires:       %{name}-openmpi = %{version}-%{release}
Requires:       openmpi-devel superlu_dist-openmpi-devel ptscotch-openmpi-devel %{blaslib}-devel

%description openmpi-devel
Development files for %name-openmpi

%package mpich
Summary:        High performance matrix preconditioners - mpich
BuildRequires:  superlu_dist-mpich-devel ptscotch-mpich-devel make
Requires:       mpich

%description mpich
Hypre is a set of matrix preconditioning libraries to aid in the solution of large systems of linear equations.

This is the mpich version.

%package mpich-devel
Summary:        Development files for %name-mpich
Requires:       %{name}-mpich = %{version}-%{release}
Requires:       mpich-devel superlu_dist-mpich-devel ptscotch-mpich-devel %{blaslib}-devel

%description mpich-devel
Development files for %name-mpich

%if %{with texlive}
%package doc
Summary:        Documentation for hypre
BuildArch:      noarch

%description doc
Documentation for hypre
%endif

%prep
%autosetup -n %name-%version -p1

find \( -name \*.[ch] -o -name \*.cxx \) -perm /=x -exec chmod 0644 {} \;

cp -a src openmpi
cp -a src mpich

%build
pushd src
export LIBS='-lsuperlu -fopenmp'
%configure --without-MPI --with-timing --with-openmp --enable-shared=yes \
           %lalibs \
           --with-superlu --with-superlu-include=%_includedir/SuperLU \
           --with-superlu-libs=superlu --with-mli \
           CFLAGS="$CFLAGS -O3"
%make_build SONAME=libHYPRE.so.%soversion
popd

%global do_mpi_build \
%configure --prefix=$MPI_HOME --with-MPI --with-MPI-include=$MPI_INCLUDE \\\
           --with-MPI-lib-dirs=$MPI_LIB --with-timing --without-openmp \\\
           %lalibs \\\
           --enable-shared=yes --with-fei --with-mli \\\
           --with-dsuperlu --with-dsuperlu-include=$MPI_INCLUDE/superlu_dist \\\
           --with-dsuperlu-libs='superlu_dist ptscotch' \\\
           CFLAGS="$CFLAGS -O3" CC=mpicc \
  %make_build SONAME=libHYPRE.so.%soversion

export LIBS='-lsuperlu_dist -lptscotch'
pushd openmpi
%_openmpi_load
%do_mpi_build
%_openmpi_unload
popd

pushd mpich
%_mpich_load
%do_mpi_build
%_mpich_unload
popd

%check
%if %{with check}
export LD_LIBRARY_PATH=$(pwd)/openmpi/hypre/lib
export OMPI_MCA_orte_allocation_required=0
%_openmpi_load
pushd openmpi/test
%make_build SONAME=libHYPRE.so.%soversion
./runtest.sh TEST_ij/*sh
popd
%endif

%install
make -C src install HYPRE_INSTALL_DIR=%{buildroot}%{_prefix} \
     HYPRE_LIB_INSTALL=%{buildroot}%{_libdir} \
     HYPRE_INC_INSTALL=%{buildroot}%{_includedir}/%{name} SONAME=libHYPRE.so.%soversion

%_openmpi_load
make -C openmpi install HYPRE_INSTALL_DIR=%{buildroot}$MPI_HOME \
     HYPRE_LIB_INSTALL=%{buildroot}$MPI_LIB \
     HYPRE_INC_INSTALL=%{buildroot}$MPI_INCLUDE/%{name} SONAME=libHYPRE.so.%soversion
%_openmpi_unload

%_mpich_load
make -C mpich install HYPRE_INSTALL_DIR=%{buildroot}$MPI_HOME \
     HYPRE_LIB_INSTALL=%{buildroot}$MPI_LIB \
     HYPRE_INC_INSTALL=%{buildroot}$MPI_INCLUDE/%{name} SONAME=libHYPRE.so.%soversion
%_mpich_unload

for l in '' mpich/lib openmpi/lib; do
  ln -s libHYPRE.so.%soversion %{buildroot}%_libdir/$l/libHYPRE.so.%somajor
done

%files
%license COPYRIGHT LICENSE-*
%doc CHANGELOG README.md
%{_libdir}/libHYPRE.so.%{somajor}*

%files devel
%{_libdir}/libHYPRE.so
%{_includedir}/%{name}

%files openmpi
%license COPYRIGHT LICENSE-*
%doc CHANGELOG README.md
%{_libdir}/openmpi/lib/libHYPRE.so.%{somajor}*

%files openmpi-devel
%{_libdir}/openmpi/lib/libHYPRE.so
%{_includedir}/openmpi-%_arch/%{name}

%files mpich
%license COPYRIGHT LICENSE-*
%doc CHANGELOG README.md
%{_libdir}/mpich/lib/libHYPRE.so.%{somajor}*

%files mpich-devel
%{_libdir}/mpich/lib/libHYPRE.so
%{_includedir}/mpich-%_arch/%{name}

%if %{with texlive}
%files doc
%license COPYRIGHT LICENSE-*
%doc CHANGELOG README.md src/examples
%doc src/docs/*.pdf src/docs/*-manual-html
%endif

%changelog
* Tue Jan 09 2024 Rebuild Robot <rebot@opencloudos.org> - 2.24.0-4
- Rebuilt for SuperLU

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 2.24.0-3
- Rebuilt for OpenCloudOS Stream 23.09

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 2.24.0-2
- Rebuilt for OpenCloudOS Stream 23.05

* Tue Apr 25 2023 Shuo Wang <abushwang@tencent.com> - 2.24.0-1
- initial build
